function [ Answer_X,  reject_X, bif_X] = BranchNBound(INTLAB_CONST_TEMP, List_X, epsilon, delta,weight)
%BranchNBound   -- This is the Branch and Bound algorithm
%               -- See Algorithm 6.1 -- 

% In order to run the code in parallel, we need to pass INTLAB_CONST to
% this function
global INTLAB_CONST;
INTLAB_CONST = INTLAB_CONST_TEMP;

% The  maximum dimension are hard coded.
% The maximum subdivision dimension is 7.
%       alpha (i=0), omega, a_1, and a_k,b_k for k=2,3 (7 dimensions)
        d=6;

% 0. We prepare the output.
Answer_X={};
reject_X={};
bif_X ={};



% 1. If list is empty, we exit. 
while ~isempty(List_X)
    
    % 2. Pop X from S
    X = List_X{end};
    List_X(end) =[];

    % We make sure to enter the while loop (created by Step 8).
    ratio =1+2*delta;
    while ratio > (1+delta )
        % We calculate the projected volume of the initial cube
        vol_initial  =Volume(X,d);
        % 3. We Prune X. 
        [ flag] = Prune( X );
        
        if flag == 1
            % 4.     flag = 1    -- Reject X
            break
        elseif flag ==2 
            % 5.    flag = 2    -- Add X to bifurcation list 
            bif_X{end+1} = X;
            break
        elseif flag ==3 
            % 6.	  flag = 3    -- Add X to answer list 
            Answer_X{end+1} =X;
            break
        else
            %     else, flag = 0.
            %
            % 7.  Check to see whether to reject cube. 
            % We calculate the maximum weighted diameter:
            % (We only weight \alpha for subdivision)
            max_diam = 2*max( [X.omega.rad , X.A(1).rad , X.A(2).rad ,X.B(2).rad,X.A(3).rad ,X.B(3).rad, weight*X.alpha.rad]);
            % If the division box is too small, then we reject.
            if ~( max_diam > epsilon)
                disp('Too small!!!')
                reject_X{end+1} =X;
                break
            else
                % 8. We check to see if we want to prune again.
                vol_final =Volume(X,d);
                ratio = vol_initial / vol_final;
                if ratio > (1+delta )
                    % We want to continue Pruning
                    continue
                else
                    % 9. We split the box into two and add the pieces to the list. 
                    % We only weight \alpha for subdivision
                    [ X1, X2 ] = Branch( X ,weight);

                    List_X{end+1} =X1;
                    List_X{end+1} =X2;
                end
            end           
        end % flag
    end % ratio-While 
    
    
end% empty list

end

