function [ X1, X2 ] = Branch( X , weight)
%BRANCH     -- See Algorithm 6.1 -- Step 9
% This function chooses to branch along one of the dimensions: 
%       alpha, omega, and a_k,b_k for k=1,2,3 (7 dimensions)
%   The output is two cubes. 

% % We display the coordinates of the box we split.
% %             disp([X.A(1) X.omega ; X.A(2) X.B(2)  ;X.A(3) X.B(3)  ])

% We create two copies of each dimension we might split. 
            w1 = X.omega;
            w2 = X.omega;
            
            A1 = X.A;
            A2 = X.A;
            
            B1 = X.B;
            B2 = X.B;
            
            alpha1 = X.alpha;
            alpha2 = X.alpha;
            
            rad0 = weight*X.alpha.rad;
            rad1 = X.omega.rad;
            rad2 = X.A(1).rad;
            rad3 = X.A(2).rad;
            rad4 = X.B(2).rad;           
            rad5 = X.A(3).rad;
            rad6 = X.B(3).rad;
            
            % We determine what the largest radii is 
            max_rad = max([rad0 rad1 rad2 rad3 rad4 rad5 rad6 ]);
            
            % We determine WHICH dimension was the largest, and split it.

            %%%% Split along 7 dimensions
            if rad0 == max_rad
                % We split alpha into 2
%                 disp('                                                  split alpha')
                alpha1 = infsup(X.alpha.inf,X.alpha.mid);
                alpha2 = infsup(X.alpha.mid,X.alpha.sup);       
            elseif rad1 == max_rad
                % We split omega into 2
%                 disp('                                                  split w')
                w1 = infsup(X.omega.inf,X.omega.mid);
                w2 = infsup(X.omega.mid,X.omega.sup);
            elseif rad2 == max_rad
                % We split epsilon into 2
%                 disp('                                                  split a1')
                A1(1) = infsup(X.A(1).inf, X.A(1).mid);
                A2(1) = infsup(X.A(1).mid, X.A(1).sup);
            elseif rad3 == max_rad
                % We split A2 into 2
%                 disp('                                                  split a2')
                A1(2) = infsup(X.A(2).inf,X.A(2).mid);
                A2(2) = infsup(X.A(2).mid,X.A(2).sup);
%             else
            elseif rad4 == max_rad
                % We split B2 into 2
%                 disp('                                                  split b2')
                B1(2) = infsup(X.B(2).inf,X.B(2).mid);
                B2(2) = infsup(X.B(2).mid,X.B(2).sup);
            elseif rad5 == max_rad
                % We split A3 into 2
%                 disp('                                                  split a3')
                A1(3) = infsup(X.A(3).inf,X.A(3).mid);
                A2(3) = infsup(X.A(3).mid,X.A(3).sup);
            else
                % We split B3 into 2
%                 disp('                                                  split b3')
                B1(3) = infsup(X.B(3).inf,X.B(3).mid);
                B2(3) = infsup(X.B(3).mid,X.B(3).sup);
            end
            
            
            % We create the two new cubes X1 and X2            
            X1 =copy(X);
            X1.alpha = alpha1;
            X1.omega=w1;
            X1.A =A1;
            X1.B =B1;
            
            X2 =copy(X);
            X2.alpha = alpha2;
            X2.omega=w2;
            X2.A =A2;
            X2.B =B2;
            
            
            % If omega was split, then we update Uw in each cube            
            if rad1 == max_rad
                % We update the U_\omega function
                X1.update   
                X2.update   
            end
            

end

