function [ tail_out ] = FTail( X )
%FTail  -- This bounds F_\infty --
%       -- See Definition 2.11(i) / Eq. (21) --
%   Output is a real column vector 1 x M.
    
% h(X)_k := 2 C_0^2 / (  (s-1) M^{s-1} (M+k+1)^s  )
%
%              + 2 C_0 \sum_{j=M-k+1}^M  |X|_j /(j+k)^s 

%  We compute  the following fraction for all -k- at once.
%
%          2 C_0^2              1
%       -------------  X    ---------
%       (s-1) M^{s-1}       (M+k+1)^s 
    
    % First product
    coeff1 = 2*(X.C0)^2 / ( (X.s-1)*(X.M)^(X.s-1) );
    % Second product
    Tail_1 = (X.M+1) + (1:X.M);
    Tail_1 = intval(transpose(Tail_1));
    Tail_1 = Tail_1.^(-X.s);
    % Product_1 times Product_2
    Tail_1 = coeff1*Tail_1;        
    
    
    
    
    
    
% We Calculate the sum:
%        2 C_0 \sum_{j=M-k+1}^M  |X|_j /( j+k)^s 

%     We write the sum as a matrix product of 
%       |X|_j with a coefficient matrix
    
    % We create a column vector for which
    %   C(j) = |X|_j. 
    C = sqrt((X.A).^2 + (X.B).^2);
    
    % We initialize the coefficient matrix.
    decayMat = X.Zeros;    
    
    % We want the k'th row of our coefficient matrix 
    % to equal '0' 
    %       for its entries between         1     --- M-k
    % to range from 1/(M+1)^s to 1/(M+k)^s 
    %       for the entries between         M-k+1 --- M 
    
    % We create a Row vector which ranges from M+1 to 2M
    Row  = X.M + (1:X.M);  
    % We take its exponent to the power -s.
    Row = 1./intval(Row)    ;
    Row=Row.^(X.s);
    
    % We insert this Row into the matrix, counting down from M. 
    %   The M'th row has Row in its entirety.
    %   The 1'st row just has an entry in the last place.
    k =X.M;
    while k>0
        % We put 'Row' into the k^th row.
        decayMat(k,:) =Row;
        % We shift 'Row' to the right. 
         Row = [0,  Row(1:end-1)];
        % Decrease counter
        k = k-1;
    end        
    % We compute the matrix product. 
    Tail_2 = 2 *X.C0* (decayMat * C);
   
    tail_out = Tail_1 + Tail_2;
    
    
end

