function [ out_R, out_I ] = Feval(X)
%FEVAL  -- This is F_M -- 
%       -- See Definition 2.3 and Eq. (9) and Eq. (16a). 
%   The output is a 2 x M column matrix 
%   representing the real and imaginary parts of F_M(X). 


%%%%%%%%%%%%%%%%%
%%%  Linear   %%%
%%%%%%%%%%%%%%%%%
linearR = ( X.UR) .* X.A               + (-(X.omega/X.alpha) * X.linear -  X.UI) .* X.B;
linearI = ( (X.omega/X.alpha)*X.linear +  X.UI ).* X.A    + (  X.UR) .* X.B;


%%%%%%%%%%%%%%%%%
%%%%    N1    %%%
%%%%%%%%%%%%%%%%%

% We calculate in Eq (9) the sum
%         k-1
% N1 =  Sum    e^{-iwj}  c_j c_{k-j}
%         j=1

% We calculate << Exp^{-i w j) c_j  >> to help with defining N1
UC_R = X.UR .* X.A  - X.B .* X.UI;
UC_I = X.A  .* X.UI + X.B .* X.UR;

% We create matricies which represent the real and imaginary parts of  C_{k-j}

% We initialize the matricies
N1A = X.Zeros;
N1B = X.Zeros;

% We create the column vectors .
a1=X.A;
b1=X.B;
% We insert the columns from left to right, shifting down every iterate.
for j=1: X.M
    % We compute the coefficient of c_j
     a1 = [0 ; a1(1:end-1)];
     b1 = [0 ; b1(1:end-1)];
    N1A(:,j)=a1;
    N1B(:,j)=b1;
end
 
N1R = (N1A * UC_R - N1B * UC_I);
N1I = (N1A * UC_I + N1B * UC_R);

%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%
%%%%    N2    %%%
%%%%%%%%%%%%%%%%%

% We calculate the truncated sum
%
%          M-k
% N2 =  Sum    lambda  c_{k+j}  c_j^*
%          j=1
%
%                           lambda = Exp^{-i w (j+k)} + Exp^{i w j}


% We create matricies which represent the real and imaginary parts of 
%        lambda * C_{k+j}
% We initialize the real/imaginary parts of this matrix
N2A =X.Zeros;
N2B =X.Zeros;
% We construct the matrix column by column. 

% This will be the     c_(j+k)              part.
a = X.A;
b = X.B;
% This will be the     Exp^{-i w (j+k)}     part.
ur = X.UR;
ui = X.UI;

for j=1: X.M-1
    % We compute the coefficient of c_j 
    %     ... the higher j is, the less it can be multiplied by

    % a&b   is from c_{j+k}
    a = [a(2:end);0];
    b = [b(2:end);0];
    % ur&ui is from e^{-i w (j+k)}
    ur(1)=[];
    ui(1)=[];
    
    lambdaR = ur + X.UR(j);
    % the minus is for it being e^{iwj} not e^{-iwj}.
    lambdaI = ui - X.UI(j); 
    
    lambdaR(X.M,1)=0;
    lambdaI(X.M,1)=0;
    
    % We do component-wise multiplication between c_(j+k) and lambda
    N2A(:,j) = lambdaR.*a - lambdaI.*b;
    N2B(:,j) = lambdaI.*a + lambdaR.*b;
end 
% We compute the dot product between 
% Matrix N2 and vector c_j^*     ----!!!! Note complex conjugate.
% This produces the real/imaginary parts of N2
N2R = (N2A * X.A + N2B * X.B);
N2I = (- N2A * X.B + N2B * X.A);

%%%%%%%%%%%%%%

% We define 
out_R = linearR +N1R +N2R;
out_I = linearI +N1I +N2I;

%%%%%%%%%%%%%%%%%%


end

