function [ flag] = Prune( X )
%PRUNE      -- This describes the Pruning algorithm 
%           -- See Algorithm 4.1 --
%   The results of the computation are interpreted as follows:
%     flag = 0    -- Continue
%     flag = 1    -- Reject X
%     flag = 2    -- Add X to bifurcation - answer list 
%	  flag = 3    -- Add X to answer list  
%  While X is not an output, it is indeed modified by this function.


    % 1. We calculate delta, a bound on the ell^1 norm of X. 
     
    delta = 2* sum( sqrt( X.A .^2 + X.B .^2) );
    
    delta = delta + (2*X.C0)/( (X.s-1)*X.M^(X.s-1)   );


    % 2. Is X in the Zero neighborhood?
    g_bound = sqrt( (1- X.omega / X.alpha )^2 + 2*( X.omega / X.alpha )*(1 - sin( X.omega)) );

    if (0 < X.alpha) && (X.alpha <= 2) && ( X.omega >= intval('1.1') ) &&  (delta < g_bound) 
        flag = 1; 
%         disp('Zero Nbd')
        return
    end   

    % 3. Is X in the bifurcation neighborhood?      
    if ( abs( X.alpha - intval('pi')/2 )<intval('0.00553') )  && abs( X.omega-intval('pi')/2 ) <= intval('0.0924')
        if delta < intval('0.18')
            flag =2;
%             disp('Bif Nbd')
            return
        end
    end   

    % 4.  We see if F is bounded away from zero on X
    [out_R,out_I] = Feval(X);
    tail_out = FTail( X );
    
    if any( (out_R.^2+out_I.^2) >  tail_out.^2)
        flag = 1;
%         disp('|F(x)| > 0')
        return
    end

 
    % 5. We calculate the Krawczyk operator, K', as in Def 3.2.  
    [ K, flag_unique ] = Krawczyk( X );
    % 6. If flag==3, then the hypothesis of Theorem 3.4 holds.
    %    That is, K \subset X (and other stuff).
    if flag_unique ==3 
        flag = 3;
        return
    end
  
    % 7.  We check to see if there are any empty intersections. 
    
    [empty_w, Intersect_omega]  = emptyintersect(K.omega,X.omega);  
    [empty_A, Intersect_A]      = emptyintersect(K.A,X.A);  
    [empty_B, Intersect_B]      = emptyintersect(K.B,X.B);  
        
    if any([empty_w ; empty_A ;  empty_B])
        % There is an empty intersection an we reject
%          disp('Krawczyk Reject')
        flag = 1;
        return;
    end
    
 
    
    
    
    %%% 8. We update X = X \cap K'(X) and we continue onward
    flag =0;
 
    X.omega = Intersect_omega;
    X.A     = Intersect_A;
    X.B     = Intersect_B;

    % We update tail bounds
    if K.C0 < sup(X.C0)
        X.C0 = K.C0;
    end

    % We update the U_\omega function
    X.update   



end

