function [ g_M ] = gM( X )
%gM      -- This defines  the function g_M  
%        -- See Definition 3.2        --
%        -- See also Definition 2.11  --
%
%   g_M      - This is a 1xM real vector

    % 0. We make useful variable definitions for the ensuing calculation
    
    % We define x_bar as a complex vector
    a_Bar       = intval(mid(X.A));
    b_Bar       = intval(mid(X.B));
    omega_Bar   = intval(mid(X.omega));

    recentered_omega = X.omega - omega_Bar;
    recentered_a = X.A - a_Bar;
    recentered_b = X.B - b_Bar;
    
    % We create a column of the coefficient norms
    H_M = sqrt(recentered_a.^2+recentered_b.^2);
    C = sqrt((X.A).^2 + (X.B).^2);
        
    
    %%%%%%%%%%%%%%%%%%%%
    % Definition 2.11  %
    %%%    g_M       %%%
    %%%%%%%%%%%%%%%%%%%%
    
    %%% In g_M^i and g_M^ii we compute a sum with |X|_j  and some
    %%% coefficients. 
    %%% The difference between the sum in g_M^i and g_M^ii is 
    %%% in one, the power is -s and in the other, the power is -(s-1).
    
    %%% We calculate this sum as a matrix-vector product, 
    %%% and we construct these matricies at the same time, and then take
    %%% the appropriate powers. 
        
        % We initialize the coefficient matrix.
        decay_M_i = X.Zeros;    
        decay_M_ii = X.Zeros;    
        
        % We want the k'th row of our coefficient matrix 
        % to equal '0' 
        %       for its entries between         1     --- M-k
        % to range from 1/(M+1)^*** to 1/(M+k)^*** 
        %       for the entries between         M-k+1 --- M 

        % We create a Row vector which ranges from M+1 to 2M, 
        Row  = X.M + (1:X.M);  
        % We take an entrywise inverse. 
        Row = 1./intval(Row)    ;
        % We take it's exponent to the appropriate power. 
        Row_i=Row.^(X.s-1);
        Row_ii=Row.^(X.s);
        
        
        % We insert this Row into the matrix, counting down from M. 
        %   The M'th row has Row in its entirety.
        %   The 1'st row just has an entry in the last place.
        k =X.M;
        while k>0
            % We put 'Row' into the k^th row.
            decay_M_i(k,:)  =Row_i;
            decay_M_ii(k,:) =Row_ii;
            % We shift 'Row' to the right. 
             Row_i  = [0,  Row_i(1:end-1)];
             Row_ii = [0,  Row_ii(1:end-1)];
            % Decrease counter
            k = k-1;
        end    
        
        
        
    % -- See  Definition 2.11  --
    %%%  %%%  %%%  %%%  %%% %
    % We calculate g_M^i   %
    %%%  %%%  %%%  %%%  %%% %
        
    % We will multiply by Delta_\omega ///  recentered _omega at the end
    
        summand_i_1 = 2*X.C0* (decay_M_i)*C;

        summand_i_2 = (X.C0^2)/( (X.s-2)*X.M^(X.s-2));
        % We divide by (M+k+1)^s for 1 <= k <= M
        summand_i_2 = summand_i_2 ./ ( intval( (X.M+2):(2*X.M+1) ).^(X.s) );
        summand_i_2 = transpose(summand_i_2);

        summand_i_3 = (X.C0^2)/( (X.s-1)*X.M^(X.s-1));
        % We divide by (M+k+1)^s-1 for 1 <= k <= M
        summand_i_3 = summand_i_3 ./ ( intval( (X.M+2):(2*X.M+1) ).^(X.s-1) );
        summand_i_3 = transpose(summand_i_3);
        
        
        gMi = recentered_omega*(summand_i_1 + summand_i_2 + summand_i_3);

    
    % -- See  Definition 2.11  --
    %%%  %%%  %%%  %%%  %%% %
    % We calculate g_M^ii    %
    %%%  %%%  %%%  %%%  %%% %
        summand_ii_1 = 4*(X.C0^2)/( (X.s-1)*X.M^(X.s-1));
        % We divide by (M+k+1)^s for 1 <= k <= M
        summand_ii_1 = summand_ii_1 ./ (intval( X.M+2:2*X.M+1).^(X.s));
        summand_ii_1 = transpose(summand_ii_1);

        summand_ii_2 = 2*X.C0*decay_M_ii*H_M;

        gMii = summand_ii_1 + summand_ii_2;

    
    %   -- See Definition 3.2  --
        % We combine gMi and gMii 
            g_M = gMi + gMii;
        % We make a real/complex vector out of gM
            g_M = [ g_M;g_M];
end

