addpath('/home/jaquette/Middle/Cube/')
%cd ..
addpath('/home/jaquette/Intlab_V10.1/')
startup
cd('/home/jaquette/Middle/')


%%% This executes the proof of Theorem 1.1

% This program needs INTLAB
% We use the library defined class "Cube"
addpath('Cube')

% This function first needs  "AP_Bounding_Functions(i)" to be ran with i=1,2,3,4
% This will output the required files below:
% % 'var_AP_Bounding_Functions_1.57-1.6.mat'
% % 'var_AP_Bounding_Functions_1.6-1.7.mat'
% % 'var_AP_Bounding_Functions_1.7-1.8.mat'
% % 'var_AP_Bounding_Functions_1.8-1.9.mat'
addpath('Data')





% List of what we do, and the relevant variables.  

% A. We import the computational results from [10, Alg. 5.1]
% B. We run Algorithm 5.7 :: InitialPrep 
% C. We do Branch and bound algorithm
% D. We perform PostVerify (Algorithm 6.3).
% E. We output results

% We start our parallel pool
parpool('local',20)
%%% In order to run INTLAB in parallel, we pass INTLAB_CONST to the workers 
global INTLAB_CONST




for i=1:4 
    
    % A. We import the computational results from [10, Alg. 5.1]
    if i == 1
        load('var_AP_Bounding_Functions_1.57-1.6.mat');
    elseif i == 2
        load('var_AP_Bounding_Functions_1.6-1.7.mat');
    elseif i == 3
        load('var_AP_Bounding_Functions_1.7-1.8.mat');
    else
        load('var_AP_Bounding_Functions_1.8-1.9.mat');
    end
    
    
    %%% Eventually the 4 lines below are redundant....
    % We create a string corresponding to alpha, for our output files
    alpha_str_1  = num2str(alpha.inf,3);
    alpha_str_2  = num2str(alpha.sup,3);
    alpha_str    = strcat('_',alpha_str_1,'-',alpha_str_2);

    

% B. We run Algorithm 5.7 :: InitialPrep 

    % The computational parameters are as follows:


    M=10;
    S =3; % ----- this is hard coded inside the function
    N= 15;


    tic
    [List_X,cubes_left_over] = InitialPrep(tmin, res, alpha, List_Q, List_Qbar, List_X_I, M,N );
    time_prep = toc 
    




% C. We do Branch and bound algorithm

    % We define the appropriate computational parameters 
    % Note: the subdivision dimenion d, is hard coded. 
    if alpha.inf < 1.6
        epsilon = 0.0001; %%%% Should be 0.0001
    else
        epsilon = 0.01;  %%%% Should be 0.01
    end
    delta = 0.5;
    weight = 8;

    % We shuffle our list to more evenly distribute the tasks among the workers
     List_X = List_X(randperm(numel(List_X)));
    % We Perform the branch and bound algorithm in parallel
     tic
    parfor k = 1: length(List_X)
        k
        [ Answer_X,  reject_X, bif_X] = BranchNBound(INTLAB_CONST, {List_X{k}} ,epsilon ,delta,weight);

        A_list{k} =Answer_X;
        R_list{k} =reject_X;
        B_list{k} =bif_X;
    end
    time_Fourier_BnB = toc


    % We recombine the answer from the Branch and Bound algorithm into 3 lists
    Answer_X = {};
    reject_X = {};
    bif_X = {};
    for k = 1:length(List_X)
        Answer_X = { Answer_X{:},A_list{k}{:}};
        reject_X = { reject_X{:},R_list{k}{:}};
        bif_X = { bif_X{:}      ,B_list{k}{:}};
    end
    




% D. We perform PostVerify (Algorithm 6.3).
    prune_rep = 5;
    tic
    [ Answer_X_2, Combined_Reject, success,I_A, I_B ] = PostVerify( Answer_X,  reject_X, bif_X,prune_rep)
    time_verify =toc; 
    
    cd('Data_New')
    save(strcat('variables',alpha_str,'.mat'));
    


% E. We output results

    % We save the variables


    
    % alpha0 = infsup(1.94,1.95); % The alpha range we are interested in looking at.
    name= strcat('out',alpha_str,'.txt');
    
    
    fileID = fopen(name,'w');




    % We output our parameter values

    
  
    

    
    fprintf(fileID,'INPUT VALUES:: \n');
    fprintf(fileID,['alpha = [' num2str(alpha.inf) ' , '  num2str(alpha.sup) ' ]\n']);
    fprintf(fileID,['subdivisions  = ' num2str(N) '\n']);
    fprintf(fileID,['M   = ' num2str(M) '\n']);
    fprintf(fileID,['S   = ' num2str(S) '\n']);
    fprintf(fileID,['N   = ' num2str(N) '\n']);
    
    fprintf(fileID,['epsilon   = ' num2str(epsilon) '\n']);
    fprintf(fileID,['delta  = ' num2str(delta) '\n']);
    fprintf(fileID,['weight  = ' num2str(weight) '\n']);
    fprintf(fileID,['Prune Rep        = ' num2str(prune_rep) '\n\n']);
    
    fprintf(fileID,'OUTPUT VALUES:: \n');
    fprintf(fileID,['success            = ' num2str(success) '\n\n']);
    
    
    if ischar(I_A)
        fprintf(fileID,['I_A = ' I_A '  \n']);
    else
        fprintf(fileID,['I_A = [' num2str(inf(I_A)) ' , ' num2str(sup(I_A)) ' ] \n']);
    end
    if ischar(I_B)
        fprintf(fileID,['I_B = ' I_B '  \n\n']);
    else
        fprintf(fileID,['I_B = [' num2str(inf(I_B)) ' , ' num2str(sup(I_B)) ' ] \n\n']);
    end
    
    fprintf(fileID,['no. bounding functions   = ' num2str(length(List_X_I)) '\n']);
    fprintf(fileID,['no. pruned cubes = ' num2str(cubes_left_over) '\n']);
    fprintf(fileID,['no. combined cubes = ' num2str(length(List_X)) '\n\n']);
    
    
    fprintf(fileID,['(sec) time_bounding_functions  = ' num2str(time_bounding_functions) '\n']);
    fprintf(fileID,['(sec) time_prep  = ' num2str(time_prep) '\n']);
    fprintf(fileID,['(sec) time_Fourier_BnB  = ' num2str(time_Fourier_BnB) '\n']);
    fprintf(fileID,['(sec) time_verify   = ' num2str(time_verify ) '\n']);

    cd .. 
end
